/* ==========================================================================
        curses.h
   --------------------------------------------------------------------------
        Copyright (C) 1987,1988,1989.  All Rights Reserved.
        Daniel J. Reynolds
        High Aspect Development Corporation
        9821 South Pulaski Road
        Evergreen Park, IL  60642   312-424-5997
   ========================================================================== */
#include <video.h>
#include <term.h>
/* -----------------------------------------
    manifest constants for curses functions
   ----------------------------------------- */
typedef unsigned short bool;
typedef unsigned short chtype;
#define reg register

#define WIN             (1)         /* possible values of WINDOW->_type */
#define PAD             (2)
#define SUBWIN          (4)
#define SUBPAD          (8)
#define OVERLAY         (1)         /* constants used for overlay() */
#define OVERWRITE       (0)         /* overwrite() and copywin() */
#define A_NORMAL        _cur_scr->_normal
#define A_UNDERLINE     _cur_scr->_underline
#define SCR_SIZE        _cur_scr->_scr_size
#define LINES           _cur_scr->_lines
#define COLS            _cur_scr->_cols
#define _TABS           _cur_scr->_tabs
#define _HALFDELAY      _cur_scr->_halfdelay
#define _DELAYOUT       _cur_scr->_delayout
#define _ENDWIN         _cur_scr->_endwin
#define _INCURSES       _cur_scr->_incurses
#define _CBREAK         _cur_scr->_cbreak
#define _CLEAR          _cur_scr->_clear
#define _RAW            _cur_scr->_raw
#define _ECHO           _cur_scr->_echo
#define _NL             _cur_scr->_nl
#define _FILLCH         _cur_scr->_fillch
#define _ERASECHAR      _cur_scr->_erasechar
#define _KILLCHAR       _cur_scr->_killchar
#define _SUBWIN         (0x0001)    /* bit flags for WINDOW->_flags */
#define _ENDLINE        (0x0002)
#define _FULLWIN        (0x0004)
#define _ACTIVE         (0x0008)
#define _SCROLLWIN      (0x0010)
#define _FLUSH          (0x0020)
#define _FULLINE        (0x0040)
#define _DELAY          (0x0080)
#define _TIMEOUT        (0x0100)
#define _STANDOUT       (0x0200)
#define _INSL           (0x0400)
#define _RESERVED       (0x0800)
#define _DELL           (0x1000)
#define _IDL            (0x2000)
#define _META           (0x4000)
#define _KEYPAD         (0x8000)
#define A_ATTRIBUTES    (0xff00)    /* masks out character data */
#define A_CHARTEXT      (0x00ff)    /* masks out attribute data */
#define A_BOLD          (0x0800)    /* attribute bit masks */
#define A_BLINK         (0x8000)
#define A_REVERSE       (revr_attr(A_NORMAL))
#define A_STANDOUT      (A_NORMAL | A_BOLD)
#define A_DIM           (A_NORMAL)
#define A_ALTCHARSET    (0x0080)
#define C_INVISIBLE     (0)         /* used for curs_set() */
#define C_NORMAL        (1)
#define C_VISIBLE       (2)
#define CALLOC_ERROR    (-1)        /* possible values of curses_error */
#define OPEN_ERROR      (-2)
#define READ_ERROR      (-3)
#define WRITE_ERROR     (-4)
#define TIMEOUT_ERROR   (-5)
#define PROGRAM_ERROR   (-6)
/* --------------------------------------------------------------------------
    The following Manifest constants define the UNIX Curses token names for
    the function keys, alternate keys and cursor control keys.  A table
    in the file named keypad.c is provided to translate the IBM scan codes
    for these keys to UNIX token values.  Note: these values are octal.
   -------------------------------------------------------------------------- */
#define KEY_BREAK       0401
#define KEY_DOWN        0402
#define KEY_UP          0403
#define KEY_LEFT        0404
#define KEY_RIGHT       0405
#define KEY_HOME        0406
#define KEY_BACKSPACE   0407
#define KEY_F0          0410
#define KEY_F1          0411
#define KEY_F2          0412
#define KEY_F3          0413
#define KEY_F4          0414
#define KEY_F5          0415
#define KEY_F6          0416
#define KEY_F7          0417
#define KEY_F8          0420
#define KEY_F9          0421
#define KEY_F10         0422
#define KEY_F11         0423
#define KEY_F12         0424
#define KEY_F13         0425
#define KEY_F14         0426
#define KEY_F15         0427
#define KEY_F16         0430
#define KEY_F17         0431
#define KEY_F18         0432
#define KEY_F19         0433
#define KEY_F20         0434
#define KEY_F21         0435
#define KEY_F22         0436
#define KEY_F23         0437
#define KEY_F24         0440
#define KEY_F25         0441
#define KEY_F26         0442
#define KEY_F27         0443
#define KEY_F28         0444
#define KEY_F29         0445
#define KEY_F30         0446
#define KEY_F31         0447
#define KEY_F32         0450
#define KEY_F33         0451
#define KEY_F34         0452
#define KEY_F35         0453
#define KEY_F36         0454
#define KEY_F37         0455
#define KEY_F38         0456
#define KEY_F39         0457
#define KEY_F40         0460
#define KEY_F41         0461
#define KEY_F42         0462
#define KEY_F43         0463
#define KEY_F44         0464
#define KEY_F45         0465
#define KEY_F46         0466
#define KEY_F47         0467
#define KEY_F48         0470
#define KEY_F49         0471
#define KEY_F50         0472
#define KEY_F51         0473
#define KEY_DL          0510
#define KEY_IL          0511
#define KEY_DC          0512
#define KEY_IC          0513
#define KEY_EIC         0514
#define KEY_CLEAR       0515
#define KEY_EOS         0516
#define KEY_EOL         0517
#define KEY_SF          0520
#define KEY_SR          0521
#define KEY_NPAGE       0522
#define KEY_PPAGE       0523
#define KEY_STAB        0524
#define KEY_CTAB        0525
#define KEY_CATAB       0526
#define KEY_ENTER       0527
#define KEY_SRESET      0530
#define KEY_RESET       0531
#define KEY_PRINT       0532
#define KEY_LL          0533
#define KEY_A1          0534
#define KEY_A3          0535
#define KEY_B2          0536
#define KEY_C1          0537
#define KEY_C3          0540
#define KEY_BTAB        0541
#define KEY_BEG         0542
#define KEY_CANCEL      0543
#define KEY_CLOSE       0544
#define KEY_COMMAND     0545
#define KEY_COPY        0546
#define KEY_CREATE      0547
#define KEY_END         0550
#define KEY_EXIT        0551
#define KEY_FIND        0552
#define KEY_HELP        0553
#define KEY_MARK        0554
#define KEY_MESSAGE     0555
#define KEY_MOVE        0556
#define KEY_NEXT        0557
#define KEY_OPEN        0560
#define KEY_OPTIONS     0561
#define KEY_PREVIOUS    0562
#define KEY_REDO        0563
#define KEY_REFERENCE   0564
#define KEY_REFRESH     0565
#define KEY_REPLACE     0566
#define KEY_RESTART     0567
#define KEY_RESUME      0570
#define KEY_SAVE        0571
#define KEY_SBEG        0572
#define KEY_SCANCEL     0573
#define KEY_SCOMMAND    0574
#define KEY_SCOPY       0575
#define KEY_SCREATE     0576
#define KEY_SDC         0577
#define KEY_SDL         0600
#define KEY_SELECT      0601
#define KEY_SEND        0602
#define KEY_SEOL        0603
#define KEY_SEXIT       0604
#define KEY_SFIND       0605
#define KEY_SHELP       0606
#define KEY_SHOME       0607
#define KEY_SIC         0610
#define KEY_SLEFT       0611
#define KEY_SMESSAGE    0612
#define KEY_SMOVE       0613
#define KEY_SNEXT       0614
#define KEY_SOPTIONS    0615
#define KEY_SPREVIOUS   0616
#define KEY_SPRINT      0617
#define KEY_SREDO       0620
#define KEY_SREPLACE    0621
#define KEY_SRIGHT      0622
#define KEY_SRSUME      0623
#define KEY_SSAVE       0624
#define KEY_SSUSPEND    0625
#define KEY_SUNDO       0626
#define KEY_SUSPEND     0627
#define KEY_UNDO        0630
/* ------------------
    attribute macros
   ------------------ */
#define wattrset(w,a)       (w->_attrs = a)
#define wattron(w,a)        (w->_attrs |= a)
#define wstandout(w)        (w->_attrs = A_STANDOUT)
#define wstandend(w)        (w->_attrs = A_NORMAL)
#define wattroff(w,a)       (w->_attrs &= ~(a))
#define attrset(a)          (stdscr->_attrs = a)
#define attron(a)           (stdscr->_attrs |= a)
#define standout()          (stdscr->_attrs = A_STANDOUT)
#define standend()          (stdscr->_attrs = A_NORMAL)
#define attroff(a)          (stdscr->_attrs &= ~(a))
#define wattrget(w)         (w->_attrs)
#define attrget()           (stdscr->_attrs)
/* ------------------------
    window location macros
   ------------------------ */
#define getyx(w,y,x)        (y=w->_cury,x=w->_curx)
#define gety(w,y)           (y=w->_cury)
#define getx(w,x)           (x=w->_curx)
#define getbegyx(w,y,x)     (y=w->_begy,x=w->_begx)
#define getbegy(w,y)        (y=w->_begy)
#define getbegx(w,x)        (x=w->_begx)
#define getmaxyx(w,y,x)     (y=w->_maxy,x=w->_maxx)
#define getmaxy(w,y)        (y=w->_maxy)
#define getmaxx(w,x)        (x=w->_maxx)
#define getmaxrc(w,r,c)     (r=w->_maxy-1,c=w->_maxx-1)
#define getmaxr(w,r)        (r=w->_maxy-1)
#define getmaxc(w,c)        (c=w->_maxx-1)
#define getsyx(y,x)         (y=curscr->_cury,x=curscr->_curx)
#define setsyx(y,x)         (curscr->_cury=y,curscr->_curx=x)
#define wsetscrreg(w,t,b)   (w->_tmarg=t,w->_bmarg=b)
#define wgetscrreg(w,t,b)   (t=w->_bmarg,b=w->_bmarg)
#define setscrreg(t,b)      (stdscr->_tmarg=t,stdscr->_bmarg=b)
#define getscrreg(t,b)      (t=stdscr->_tmarg,b=stdscr->_bmarg)
#define getscroll()         (stdscr->_scroll)
#define wgetscroll(w)       (w->_scroll)
/* -------------------
    i/o option macros
   ------------------- */
#define scrollok(w,bf)  (w->_scroll = bf)
#define leaveok(w,bf)   (w->_leave = bf)
#define clearok(w,bf)   (w->_clear = bf)
#define idlok(w,bf)     (bf ? (w->_flags |= _IDL)    : (w->_flags &= ~_IDL))
#define meta(w,bf)      (bf ? (w->_flags |= _META)   : (w->_flags &= ~_META))
#define keypad(w,bf)    (bf ? (w->_flags |= _KEYPAD) : (w->_flags &= ~_KEYPAD))
#define nodelay(w,bf)   (bf ? (w->_flags |= _DELAY)  : (w->_flags &= ~_DELAY))
#define cbreak()        (_CBREAK = TRUE)
#define nocbreak()      (_CBREAK = FALSE)
#define raw()           (_RAW = TRUE)
#define noraw()         (_RAW = FALSE)
#define echo()          (_ECHO = TRUE)
#define noecho()        (_ECHO = FALSE)
#define nl()            (_NL = TRUE)
#define nonl()          (_NL = FALSE)
#define halfdelay(tens) (_HALFDELAY = tens)
/* -------------
    move macros
   ------------- */
#define mvaddch(y,x,c)      (wmove(stdscr,y,x) ? waddch(stdscr,c) : ERR)
#define mvwaddch(w,y,x,c)   (wmove(w,y,x) ? waddch(w,c) : ERR)
#define mvaddstr(y,x,s)     (wmove(stdscr,y,x) ? waddstr(stdscr,s) : ERR)
#define mvwaddstr(w,y,x,s)  (wmove(w,y,x) ? waddstr(w,s) : ERR)
#define mvdelch(y,x)        (wmove(stdscr,y,x) ? wdelch(stdscr),OK : ERR)
#define mvwdelch(w,y,x)     (wmove(w,y,x) ? wdelch(w),OK : ERR)
#define mvinsch(y,x,c)      (wmove(stdscr,y,x) ? winsch(stdscr,c) : ERR)
#define mvwinsch(w,y,x,c)   (wmove(w,y,x) ? winsch(w,c) : ERR)
#define mvinch(y,x)         (wmove(stdscr,y,x) ? winch(stdscr) : ERR)
#define mvinchtx(y,x)       (wmove(stdscr,y,x) ? winchtx(stdscr) : ERR)
#define mvinattr(y,x)       (wmove(stdscr,y,x) ? winattr(stdscr) : ERR)
#define mvwinch(w,y,x)      (wmove(w,y,x) ? winch(w) : ERR)
#define mvwinchtx(w,y,x)    (wmove(w,y,x) ? winchtx(w) : ERR)
#define mvwinattr(w,y,x)    (wmove(w,y,x) ? winattr(w) : ERR)
#define mvgetch(y,x)        (wmove(stdscr,y,x) ? wgetch(stdscr) : ERR)
#define mvwgetch(w,y,x)     (wmove(w,y,x) ? wgetch(w) : ERR)
#define mvgetstr(y,x,s)     (wmove(stdscr,y,x) ? wgetstr(stdscr,s) : ERR)
#define mvwgetstr(w,y,x,s)  (wmove(w,y,x) ? wgetstr(w,s) : ERR)
/* ------------------------
    standard screen macros
   ------------------------ */
#define refresh()           wrefresh(stdscr)
#define addch(c)            waddch(stdscr,c)
#define move(y,x)           wmove(stdscr,y,x)
#define addstr(s)           waddstr(stdscr,s)
#define clrtoeol()          wclrtoeol(stdscr)
#define clrtobot()          wclrtobot(stdscr)
#define erase()             werase(stdscr)
#define clear()             wclear(stdscr)
#define delch()             wdelch(stdscr)
#define deleteln()          wdeleteln(stdscr)
#define insch(c)            winsch(stdscr,c)
#define insertln()          winsertln(stdscr)
#define inch()              winch(stdscr)
#define inchtx()            winchtx(stdscr)
#define inattr()            winattr(stdscr)
#define getch()             wgetch(stdscr)
#define getstr(s)           wgetstr(stdscr,s)
#define cpystr(a,b,c,d)     w_cpystr(stdscr,a,b,c,d)
#define echochar(c)         (waddch(stdscr,c) ? wrefresh(stdscr) : ERR)
/* -------------
    misc macros
   ------------- */
#define def_prog_mode()     v_save(_cur_scr->_video->v_prog);
#define reset_prog_mode()   v_restore(_cur_scr->_video->v_prog);
#define def_shell_mode()    v_save(_cur_scr->_video->v_shell);
#define reset_shell_mode()  v_restore(_cur_scr->_video->v_shell);
#define beep()              v_sound(880,50)
#define wclear(w)           ((werase(w),clearok(w,TRUE)),w->_flags &= ~_ACTIVE)
#define mvcur(or,oc,nr,nc)  (is_ansi()?(ansi_cup(nr,nc),OK):(c_set(nr,nc),OK))
#define slk_refresh()       wrefresh(_slkwin)
#define slk_noutrefresh()   wnoutrefresh(_slkwin)
#define slk_clear()         wclear(_slkwin)
#define slk_restore()       (slk_touch(),wrefresh(_slkwin))
#define slkattrset(a)       (_slkwin->_attrs = a)
#define winch(w)            (w->_y[w->_cury][w->_curx])
#define winchtx(w)          ((w->_y[w->_cury][w->_curx]) & A_CHARTEXT)
#define winattr(w)          ((w->_y[w->_cury][w->_curx]) & A_ATTRIBUTES)
#define delay_output(ms)    (_DELAYOUT = ms)
#define has_ic()            (FALSE)
#define has_il()            (FALSE)
#define isendwin()          (_ENDWIN)
#define baudrate()          (9600)
#define erasechar()         (_ERASECHAR)
#define killchar()          (_KILLCHAR)
#define wechochar(w,c)      (waddch(w,c) ? wrefresh(w) : ERR)
#define longname()          (cur_term->_longname)
#define unctrl(ch)          (_unctrl[(unsigned) ch])
#define pechochar(p,c)  (waddch(p,c) ? prefresh(p,p->_pminrow,p->_pmincol,\
                         p->_sminrow,p->_smincol,p->_smaxrow,p->_smaxcol):ERR)
#define overlay(s,d)    (copywin(s,d,0,0,0,0, \
                        ((d->_maxy-1 < s->_maxy-1) ? d->_maxy-1 : s->_maxy-1), \
                        ((d->_maxx-1 < s->_maxx-1) ? d->_maxx-1 : s->_maxx-1),1))
#define overwrite(s,d)  (copywin(s,d,0,0,0,0, \
                        ((d->_maxy-1 < s->_maxy-1) ? d->_maxy-1 : s->_maxy-1), \
                        ((d->_maxx-1 < s->_maxx-1) ? d->_maxx-1 : s->_maxx-1),0))
/* -------------------------------------
    macro extensions to standard curses
   ------------------------------------- */
#define w_fill(c)           (w->_fill = c)
#define w_home(w)           (w->_curx = 0)
#define w_end(w)            (w->_curx = w->_maxx - 1)
#define w_top(w)            (w->_cury = w->_curx = 0)
#define w_bottom(w)         (w->_cury = w->_maxy - 1,w->_curx = 0)
#define w_up(w)             (--w->_cury < 0 ? ++w->_cury : OK)
#define w_down(w)           (++w->_cury == w->_maxy ? --w->_cury : OK)
#define w_left(w)           (--w->_curx < 0 ? ++w->_curx : OK)
#define w_right(w)          (++w->_curx == w->_maxx ? --w->_curx : OK)
#define chgatt(a,b,c,d,e)   w_chgatt(stdscr,a,b,c,d,e)
#define drbox(a,b,c,d)      w_drbox(stdscr,a,b,c,d)
#define movblk(a,b,c,d,e,f) w_movblk(stdscr,a,b,c,d,e,f)
#define fillch(a,b,c,d,e)   w_fillch(stdscr,a,b,c,d,e)
#define set_tabs(i)         (_TABS = i)
#define get_tabs()          (_TABS)
#define set_fill(c)         (_FILLCH = c)
#define get_fill()          (_FILLCH)
#define set_color(f,b)      (A_NORMAL = attr(f,b))
/* --------------------------------------------------------------------------
        Since every implementation of curses is free to define the
        WINDOW structure, direct references to the WINDOW members is
        NOT portable.  Where possible, the same names have been given
        structure members, however, there is no guarantee that these
        members will be the same data types in other implementations.
   -------------------------------------------------------------------------- */
struct _win_st {                /* CURSES WINDOW STRUCTURE */
    struct _win_st * _subwin;   /* pointer to subwindow */
    struct _win_st * _next;     /* pointer to next window in refresh chain */
    chtype ** _y;           /* pointer to line arrays */
    short  *  _firstch;
    short  *  _lastch;
    uint    _flags;         /* window bit flags */
    int     _type;          /* window type */
    chtype  _fill;          /* char used to clear window */
    chtype  _attrs;         /* attribute used to clear window */
    bool    _clear;         /* clear screen & repaint flag */
    bool    _leave;         /* leave cursor flag */
    bool    _scroll;        /* scrolling allowed flag */
    int     _cury,_curx;    /* logical cursor row & col */
    int     _maxy,_maxx;    /* number of rows & cols */
    int     _begy,_begx;    /* absolute screen location */
    int     _pminrow,_pmincol; /* last pad refresh - pad upper left */
    int     _sminrow,_smincol; /* last pad refresh - screen start coordinates */
    int     _smaxrow,_smaxcol; /* last pad refresh - screen end coordinates */
    int     _tmarg,_bmarg;  /* defines scroll region */
};
typedef struct _win_st WINDOW;
struct _screen {
    TERMINAL * _term;
    VIDEO    * _video;
    WINDOW   * _stdscr;
    WINDOW   * _curscr;
    WINDOW   * _slkwin;
    chtype   * _this_refresh;
    chtype   * _last_refresh;
    struct  _screen  * _next;
    chtype  _normal;            /* current normal attribute */
    chtype  _underline;         /* current underline attribute */
    int     _lines;             /* number of curscr lines */
    int     _cols;              /* number of curscr columns */
    int     _scr_size;          /* screen size = lines * columns */
    int     _tabs;              /* default tab stops */
    int     _halfdelay;         /* tenths of a second to wait for input */
    int     _delayout;          /* miliseconds to delay output */
    bool    _endwin;            /* flag indicating endwin has been called */
    bool    _incurses;          /* flag indicating In Curses Mode */
    bool    _cbreak;            /* boolean flag for cbreak mode */
    bool    _clear;             /* boolean flag for clearok() */
    bool    _raw;               /* boolean flag for raw mode */
    bool    _echo;              /* boolean flag for echo mode */
    bool    _nl;                /* boolean flag for nl mode */
    chtype  _fillch;            /* global fill character */
    chtype  _erasechar;         /* global erase character */
    chtype  _killchar;          /* global kill character */
};
typedef struct _screen SCREEN;
/* -------------------------
    curses global variables
   ------------------------- */
extern int _cerror;             /* error number for last error */
extern int _key_pad[];          /* UNIX keypad translation translation */
extern char * Release;          /* release number */
extern char * Def_term;         /* default terminal */
extern char * ibm_keys[];       /* ASCII strings for non-printable keys */
extern char * unix_keys[];      /* ASCII strings for keypad key tokens */
extern char * _unctrl[];        /* array of character strings for unctrl */
extern char   term_path[];      /* TERMINFO path default to c:\terminfo */
extern WINDOW * stdscr;         /* default standard screen */
extern WINDOW * curscr;         /* current virtual screen */
extern WINDOW * _slkwin;        /* soft function key label window */
extern SCREEN * _initscr;       /* initial SCREEN requested */
extern SCREEN * _cur_scr;       /* current SCREEN - TERMINAL definition */
extern int (*x_iskey)(void);    /* pointers to user supplied i/o functions */
extern int (*x_getkey)(void);   /* called by db_CURSES whenever Extended */
extern int (*x_output)(int,int,chtype);     /* ANSI Driver is enabled */
extern VIDEO * (*x_init)(SCREEN *,int *);   /* XANSI Video initialization */
extern void (*x_reset_prog)(SCREEN *);  /* reset_prog_mode() equivalent */
extern void (*x_reset_shell)(SCREEN *); /* reset_shell_mode() equivalent */
/* -----------------------------------
    Alternate Character Set Variables
   ----------------------------------- */
extern chtype ACS_ULCORNER;   /* upper left corner */
extern chtype ACS_LLCORNER;   /* lower left corner */
extern chtype ACS_URCORNER;   /* upper right corner */
extern chtype ACS_LRCORNER;   /* lower right corner */
extern chtype ACS_RTEE;       /* right tee */
extern chtype ACS_LTEE;       /* left tee */
extern chtype ACS_BTEE;       /* bottom tee */
extern chtype ACS_TTEE;       /* top tee */
extern chtype ACS_HLINE;      /* horizontal line */
extern chtype ACS_VLINE;      /* vertical line */
extern chtype ACS_PLUS;       /* plus */
extern chtype ACS_S1;         /* scan line 1 */
extern chtype ACS_S9;         /* scan line 9 */
extern chtype ACS_DIAMOND;    /* diamond */
extern chtype ACS_CKBOARD;    /* checker board */
extern chtype ACS_DEGREE;     /* degree symbol */
extern chtype ACS_PLMINUS;    /* plus/minus */
extern chtype ACS_BULLET;     /* bullet */
extern chtype ACS_LARROW;     /* arrow pointing left */
extern chtype ACS_RARROW;     /* arrow pointing right */
extern chtype ACS_DARROW;     /* arrow pointing down */
extern chtype ACS_UARROW;     /* arrow pointing up */
extern chtype ACS_BOARD;      /* board of squares */
extern chtype ACS_LANTERN;    /* lantern sysmbol */
extern chtype ACS_BLOCK;      /* solid square block */
/* ---------------------
    function prototypes
   --------------------- */
extern SCREEN * newterm(char *,FILE *,FILE *);
extern SCREEN * set_term(SCREEN *);
extern WINDOW * initscr(void);
extern WINDOW * newwin(int ,int ,int ,int );
extern WINDOW * newpad(int ,int );
extern WINDOW * subwin(WINDOW *,int ,int ,int ,int );
extern WINDOW * subpad(WINDOW *,int ,int ,int ,int );
extern char * slk_label(int);
extern char * keyname(chtype);
extern int  wrefresh(WINDOW *);
extern int  prefresh(WINDOW *,int ,int ,int ,int ,int ,int );
extern int  scr_dump(char *);
extern int  scr_restore(char *);
extern int  ripoffline(int ,int (*)(WINDOW *,int));
extern void endwin(void);
extern void savetty(void);
extern void resetty(void);
extern int  doupdate(void);
extern void wclrtoeol(WINDOW *);
extern void wclrtobot(WINDOW *);
extern int  wmove(WINDOW *,int ,int );
extern int  waddstr(WINDOW *,char *);
extern int  curs_set(int);
extern void box(WINDOW *,chtype,chtype);
extern int  winsch(WINDOW *,chtype);
extern void wdelch(WINDOW *);
extern void winsertln(WINDOW *);
extern void wdeleteln(WINDOW *);
extern int  scroll(WINDOW *);
extern int  mvwin(WINDOW *,int ,int );
extern int  wgetch(WINDOW *);
extern int  wgetstr(WINDOW *,char *);
extern int  wnoutrefresh(WINDOW *);
extern int  pnoutrefresh(WINDOW *,int ,int ,int ,int ,int ,int );
extern int  delwin(WINDOW *);
extern void ungetch(chtype);
extern void flash(void);
extern void napms(int);
extern void flushinp(void);
extern void werase(WINDOW *);
extern int  waddch(WINDOW *,chtype);
extern void slk_init(int);
extern void slk_set(int,char *,int);
extern void slk_touch(void);
extern int  printw(char *,...);
extern int  mvprintw(int ,int ,char *,...);
extern int  wprintw(WINDOW *,char *,...);
extern int  mvwprintw(WINDOW *,int ,int ,char *,...);
#ifndef LATTICE
extern int  vwprintw(WINDOW *,char *,va_list);
#endif
extern int  scanw(char *,...);
extern int  mvscanw(int ,int ,char *,...);
extern int  wscanw(WINDOW *,char *,...);
extern int  mvwscanw(WINDOW *,int ,int ,char *,...);
#ifndef LATTICE
extern int  vwscanw(WINDOW *,char *,va_list);
#endif
extern int  copywin(WINDOW *,WINDOW *,int ,int ,int ,int ,int ,int ,int);
extern int  putwin(WINDOW *,char *);
extern int  getwin(WINDOW *,char *);
extern int  putpad(WINDOW *,char *);
extern WINDOW * getpad(char *,int);
extern void whline(WINDOW *,int ,int ,int );
extern void wvline(WINDOW *,int ,int ,int );
/* ---------------------------
    extentions to UNIX curses
   --------------------------- */
extern char * w_cpystr(WINDOW *,int ,int ,int ,char *);
extern void w_drbox(WINDOW *,int ,int ,int ,int );
extern int  w_chgatt(WINDOW *,int ,int ,int ,int ,chtype);
extern int  w_fillch(WINDOW *,int ,int ,int ,int ,chtype);
extern int  w_movblk(WINDOW *,int ,int ,int ,int ,int ,int );
extern int  w_strr(WINDOW *,int );
extern int  w_strl(WINDOW *,int );
extern int  w_travel(WINDOW *,int ,int );
extern int  w_scroll(WINDOW *,WINDOW *,char *[]);
extern int  w_select(WINDOW *,WINDOW *,char *[]);
extern int  w_edit(WINDOW *);
extern int  w_help(char **);
extern int  w_dump(WINDOW *,char *);
extern int  w_restore(WINDOW *,char *);
extern void w_setacs(int);
extern void w_setbox(chtype,chtype);

