/* ==========================================================================
        edit.c
   --------------------------------------------------------------------------
        Copyright (C) 1989.  Daniel J. Reynolds.  All Rights Reserved.
        High Aspect Development Corporation
        9821 South Pulaski Road
        Evergreen Park, IL  60642   312-424-5997
   ========================================================================== */

#include <curses.h>

char * intro_text[] = {         /* Text for intoduction screen */

"                                                                            ",
"    This program implements a trivial text editor using the db_CURSES       ",
"    library.  It is intended to show how the db_CURSES functions perform    ",
"    and serve as an example of how a Curses program operates.  Source       ",
"    for this program is included for you to examine.                        ",
"                                                                            ",
"    The bottom line of the screen contains the Curses soft function key     ",
"    labels.  Pressing [F-1] will pop up a help screen listing all the       ",
"    functions available.  Pressing [F8] quits the editor.                   ",
"                                                                            ",
"    Pressing [F-2] and [F-3] will allow you to change foreground &          ",
"    background colors. [F-7] Clears the current window.  [F-4] toggles      ",
"    attributes.  [F-6] will allow you to toggle between two different       ",
"    editing windows.  [F-5] will allow you to toggle between the video      ",
"    drivers and judge performance.  The rest of the keystrokes and          ",
"    functions should be self explainatory.                                  ",
"                                                                            ",
"    Have fun!  - press any key                                              ",
"                                                                            ",
NULL,
};

char * help_text[] = {          /* Text for help screen */

"                                                                            ",
"    F-1     This Help Window            HOME    Cursor to Start of Line     ",
"    F-2     Foreground Color            END     Cursor to End of Line       ",
"    F-3     Background Color            PG_UP   Cursor to Top of Page       ",
"    F-4     Attribute                   PG_DN   Cursor to Bottom of Page    ",
"    F-5     Video Driver                UP      Cursor Up One Line          ",
"    F-6     Change Window               DOWN    Cursor Down One Line        ",
"    F-7     Clear Screen                LEFT    Cursor Left One Column      ",
"    F-8     Quit                        RIGHT   Cursor Right One Column     ",
"    F-9     Warm Boot                   INSERT  Toggle Insert Mode          ",
"    ALT-B   Clear To Bottom             INSERT  Toggle Insert Mode          ",
"    ALT-C   Alternate Char Set          DELETE  Delete Character at Cursor  ",
"    ALT-D   Delete Line                 BACKSP  Delete Character to Left    ",
"    ALT-I   Insert Line                 TAB     Move Tab Spaces to Right    ",
"    ALT-L   Clear To EOL                ENTER   Next Line                   ",
"    ALT-S   Toggle Scrolling                                                ",
"    ALT-R   Read Window From Disk       ALT-G   Get Window in File          ",
"    ALT-W   Write Window to Disk        ALT-P   Put Window from File        ",
"                                                                            ",
"                                                                            ",
"    -  Press any key to return -                                            ",
NULL,
};

char *colors[] = {      /* Color names for status line */
    "BLACK",
    "BLUE",
    "GREEN",
    "CYAN",
    "RED",
    "MAGENTA",
    "BROWN",
    "WHITE",
};

void main(void);
void edit(void);
void help(void);
void intro(void);

/* -------------------------------------------------------------------------- */
void main () {

    break_on(&break_flag);      /* trap out ount CTL-C CTL-BREAK */
    caltd_off(&caltd_flag);     /* trap our own CTL-ALT-DELETE */
    criter_on(&critr_flag);     /* handle our own critical errors */
    slk_init(1);                /* initialize soft labels */
    initscr();
    attr_trans = FALSE;
    noecho();
    raw();
    slk_set(1,"F1 HELP ",0);     /* set up soft label names */
    slk_set(2,"F2 FORE ",0);
    slk_set(3,"F3 BACK ",0);
    slk_set(4,"F4 ATTR ",0);
    slk_set(5,"F5 DRVR ",0);
    slk_set(6,"F6 WIND ",0);
    slk_set(7,"F7 CLR  ",0);
    slk_set(8,"F8 QUIT ",0);
    edit();                     /* edit funcion */
    break_off();                /* restore DOS CTL-BREAK handler */
    caltd_on();                 /* restore DOS CTL_ALT_DELETE */
    criter_off();               /* restore DOS critical error handler */
    endwin();
}
/* -------------------------------------------------------------------------- */
void edit() {

char    *vs,*wind;
int     key,insert,altchar,row,col,attrib,attr,vidf,foreg,backg,scrll;
WINDOW  *outer,*window,*status,*w1,*w2;

    set_color(c_WHITE,c_BLACK);         /* Normal White on Black */
    slk_touch();                        /* Adds labels to slk window */
    status = newwin(1,80,0,0);          /* Set up Status line */
    outer  = newwin(LINES-1,COLS,1,0);  /* Outer window for border */
    w1 = newwin(LINES-3,COLS-2,2,1);    /* Edit window 1 */
    w2 = newwin(LINES-3,COLS-2,2,1);    /* Edit window 2 */
    box(outer,box_typ[BOX_DOUBLE].v,box_typ[BOX_DOUBLE].h);
    wrefresh(outer);
    wsetscrreg(w1,0,20);                /* Set up scroll regions */
    wsetscrreg(w2,0,20);
    scrll = TRUE;                       /* Scrooling initially enabled */
    scrollok(w1,scrll);
    scrollok(w2,scrll);
    wmove(w1,0,0);                      /* Start at origins of windows */
    wmove(w2,0,0);
    foreg = 7;                          /* Keeps track of color attributes */
    backg = 0;
    attr = 0;
    insert = altchar = FALSE;           /* Insert mode & Alt Char mode off */
    vs = "RAM";                         /* RAM Video Driver default */
    vidf = 0;
    window = w1;                        /* Window 1 initially the default v*/
    wind = "1";
    intro();                            /* Display intoduction */
    for (;;) {                          /* Loop forever */
        set_color(foreg << 8,backg << 8);   /* Set up color & attributes */
        switch (attr) {
            case    0   :
                attrib = A_NORMAL;
                break;
            case    1   :
                attrib = A_REVERSE;
                break;
            case    2   :
                attrib = A_NORMAL | A_BOLD;
                break;
            case    3   :
                attrib = A_NORMAL | A_BLINK;
                break;
            case    4   :
                attrib = A_UNDERLINE;
                break;
        }
        wattrset(window,attrib);
        if (altchar)                        /* Turn Alt Char if needed */
            wattron(window,A_ALTCHARSET);
        (void)(insert ? curs_set(C_VISIBLE) : curs_set(C_NORMAL));
        getyx(window,row,col);              /* STATUS LINE UPDATE */
        wmove(status,0,0);
        wprintw(status,"ROW: %2d COL: %2d ",row,col);
        if (insert)
            waddstr(status,"INSERT ");
        else
            waddstr(status,"OVERWR ");
        wattrset(status,attrib);
        if (altchar)
            wattroff(status,A_ALTCHARSET);
        waddstr(status,"ATTRIB");
        set_color(c_WHITE,c_BLACK);
        wattrset(status,A_NORMAL);
        set_color(foreg << 8,backg << 8);
        if (altchar)
            waddstr(status," ALTCHAR ");
        else
            waddstr(status,"         ");
        wprintw(status,"WIND: %s ",wind);
        wprintw(status,"DRIVER: %s ",vs);
        wprintw(status,"%7s - %7s",colors[foreg],colors[backg]);
        slk_noutrefresh();          /* Soft label window */
        wnoutrefresh(status);       /* Status line window */
        wnoutrefresh(window);       /* Current editing window */
        doupdate();                 /* Update the screen */
        key = wgetch(window);
        switch (key) {
            case    ALT_C   :       /* Toggles Alt Char Set */
                altchar = (altchar ? (wattroff(window,A_ALTCHARSET),FALSE)
                                   : (wattron(window,A_ALTCHARSET),TRUE));
                break;
            case    ALT_S   :       /* Toggles scrolling */
                scrll = (scrll ? FALSE : TRUE);
                scrollok(window,scrll);
                break;
            case    ALT_D   :       /* Delete line */
                wdeleteln(window);
                break;
            case    ALT_B   :       /* Clear to bottom of window */
                wclrtobot(window);
                break;
            case    ALT_I   :       /* Insert line */
                winsertln(window);
                break;
            case    ALT_L   :       /* Clear to end of line */
                wclrtoeol(window);
                break;
            case    ALT_R   :       /* Restore previously dumped window */
                w_restore(window,"dump.win");
                break;
            case    ALT_W   :       /* Save current window in disk file */
                w_dump(window,"dump.win");
                break;
            case    ALT_P   :       /* Put Window in Text File */
                putwin(window,"window.txt");
                break;
            case    ALT_G   :       /* Get Window from Text File */
                getwin(window,"window.txt");
                break;
            case    HOME        :   /* Go to start of current line */
                w_home(window);
                break;
            case    END         :   /* Go to end of current line */
                w_end(window);
                break;
            case    UP_ARROW    :   /* Go up 1 line */
                w_up(window);
                break;
            case    DOWN_ARROW  :   /* Go down 1 line */
                w_down(window);
                break;
            case    LT_ARROW    :   /* Go left 1 column */
                w_left(window);
                break;
            case    RT_ARROW    :   /* Go right 1 column */
                w_right(window);
                break;
            case    PAGE_UP     :   /* Go to top of page */
                w_top(window);
                break;
            case    PAGE_DOWN   :   /* Go to bottom of page */
                w_bottom(window);
                break;
            case    INSERT      :   /* Toggle insert mode
                                       and set cursor appropriately */
                insert = (insert ? (curs_set(C_NORMAL),FALSE)
                                 : (curs_set(C_VISIBLE),TRUE));
                break;
            case    DELETE      :   /* Delete character at cursor */
                wdelch(window);
                break;
            case    F1      :       /* Display Help Screen */
                help();
                break;
            case    F2      :       /* Change to next foreground color */
                if (++foreg > 7)
                    foreg = 0;
                break;
            case    F3      :       /* Change to next background color */
                if (++backg > 7)
                    backg = 0;
                break;
            case    F4      :       /* Change to next attribute */
                if (++attr == 5)
                    attr = 0;
                break;
            case    F5      :       /* Change to next Video Driver */
                if (vidf == 0) {       /* Change to BIOS Driver */
                    vidf = 1;
                    vs = "BIOS";
                    set_bios();
                }
                else if (vidf == 1) {  /* Change to ANSI Driver */
                    vidf = 2;
                    vs = "ANSI";
                    set_ansi();
                }
                else {  /* Change to RAM Driver */
                    vs = "RAM ";
                    vidf = 0;
                    set_ram();
                }
                break;
            case    F6      :           /* Toggle Editing Windows */
                if (window == w1) {
                    wind = "2";
                    window = w2;
                }
                else {
                    wind = "1";
                    window = w1;
                }
                break;
            case    F7      :           /* Erase Current Window */
                werase(window);
                wmove(window,0,0);
                break;
            case    F8      :           /* Quit and Exit */
                delwin(w1);             /* Delete all windows */
                delwin(w2);
                delwin(outer);
                delwin(status);
                delwin(_slkwin);
                doupdate();
                return;
            case    F9  :               /* Do a warm reboot */
                d_reboot();
            default    :                /* Otherwise display the character */
                if ((key <= 0x7f) && (key >= 0))
                    (void)(insert ? winsch(window,key) : wechochar(window,key));
        }
    }
}
/* -------------------------------------------------------------------------- */
void help() {

WINDOW  * hw;
int i;

    for (i=0;help_text[i] != NULL;++i);     /* Count the lines */
    hw = newwin(i,76,2,2);
    wattrset(hw,A_REVERSE);
    wmove(hw,0,0);
    for (i=0;help_text[i] != NULL;++i)      /* Display the lines */
        waddstr(hw,help_text[i]);
    wrefresh(hw);
    wgetch(hw);
    delwin(hw);
    return;
}
/* -------------------------------------------------------------------------- */
void intro() {

WINDOW  * iw;
int i;

    for (i=0;intro_text[i] != NULL;++i);    /* Count lines */
    iw = newwin(i,76,3,2);
    wattrset(iw,A_REVERSE);
    wmove(iw,0,0);
    for (i=0;intro_text[i] != NULL;++i)     /* Display the lines */
        waddstr(iw,intro_text[i]);
    wrefresh(iw);
    wgetch(iw);                             /* Any key will do */
    delwin(iw);
    return;
}

